@@ -87,7 +87,7 @@
 #include "config.h"
 #endif
 
-#include "clutter-gesture-action.h"
+#include "clutter-gesture-action-private.h"
 
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
@@ -103,6 +103,7 @@ typedef struct
 {
   ClutterInputDevice *device;
   ClutterEventSequence *sequence;
+  ClutterEvent *last_event;
 
   gfloat press_x, press_y;
   gint64 last_motion_time;
@@ -122,6 +123,8 @@ struct _ClutterGestureActionPrivate
   guint actor_capture_id;
   gulong stage_capture_id;
 
+  ClutterGestureTriggerEdge edge;
+
   guint in_gesture : 1;
 };
 
@@ -151,6 +154,7 @@ gesture_register_point (ClutterGestureAction *action, ClutterEvent *event)
   g_array_set_size (priv->points, priv->points->len + 1);
   point = &g_array_index (priv->points, GesturePoint, priv->points->len - 1);
 
+  point->last_event = clutter_event_copy (event);
   point->device = clutter_event_get_device (event);
 
   clutter_event_get_coords (event, &point->press_x, &point->press_y);
@@ -209,6 +213,21 @@ gesture_unregister_point (ClutterGestureAction *action, gint position)
   g_array_remove_index (priv->points, position);
 }
 
+static gint
+gesture_get_threshold (ClutterGestureAction *action)
+{
+  gint threshold;
+  ClutterSettings *settings = clutter_settings_get_default ();
+  g_object_get (settings, "dnd-drag-threshold", &threshold, NULL);
+  return threshold;
+}
+
+static void
+gesture_point_unset (GesturePoint *point)
+{
+  clutter_event_free (point->last_event);
+}
+
 static void
 cancel_gesture (ClutterGestureAction *action)
 {
@@ -225,6 +244,40 @@ cancel_gesture (ClutterGestureAction *action)
 
   actor = clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
   g_signal_emit (action, gesture_signals[GESTURE_CANCEL], 0, actor);
+
+  g_array_set_size (action->priv->points, 0);
+}
+
+static gboolean
+begin_gesture (ClutterGestureAction *action,
+               ClutterActor         *actor)
+{
+  ClutterGestureActionPrivate *priv = action->priv;
+  gboolean return_value;
+
+  priv->in_gesture = TRUE;
+
+  if (!CLUTTER_GESTURE_ACTION_GET_CLASS (action)->gesture_prepare (action, actor))
+    {
+      cancel_gesture (action);
+      return FALSE;
+    }
+
+  /* clutter_gesture_action_cancel() may have been called during
+   * gesture_prepare(), check that the gesture is still active. */
+  if (!priv->in_gesture)
+    return FALSE;
+
+  g_signal_emit (action, gesture_signals[GESTURE_BEGIN], 0, actor,
+                 &return_value);
+
+  if (!return_value)
+    {
+      cancel_gesture (action);
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 static gboolean
@@ -234,7 +287,7 @@ stage_captured_event_cb (ClutterActor       *stage,
 {
   ClutterGestureActionPrivate *priv = action->priv;
   ClutterActor *actor;
-  gint position;
+  gint position, drag_threshold;
   gboolean return_value;
   GesturePoint *point;
   gfloat motion_x, motion_y;
@@ -258,7 +311,6 @@ stage_captured_event_cb (ClutterActor       *stage,
         if (!(mods & CLUTTER_BUTTON1_MASK))
           {
             cancel_gesture (action);
-            gesture_unregister_point (action, position);
             return CLUTTER_EVENT_PROPAGATE;
           }
       }
@@ -272,43 +324,24 @@ stage_captured_event_cb (ClutterActor       *stage,
       if (priv->points->len < priv->requested_nb_points)
         return CLUTTER_EVENT_PROPAGATE;
 
+      drag_threshold = gesture_get_threshold (action);
+
       if (!priv->in_gesture)
         {
-          gint drag_threshold;
-          ClutterSettings *settings = clutter_settings_get_default ();
+          /* Wait until the drag threshold has been exceeded
+           * before starting _TRIGGER_EDGE_AFTER gestures. */
+          if (priv->edge == CLUTTER_GESTURE_TRIGGER_EDGE_AFTER &&
+              (fabsf (point->press_y - motion_y) < drag_threshold) &&
+              (fabsf (point->press_x - motion_x) < drag_threshold))
+            return CLUTTER_EVENT_PROPAGATE;
 
-          g_object_get (settings,
-                        "dnd-drag-threshold", &drag_threshold,
-                        NULL);
-
-          if ((ABS (point->press_y - motion_y) >= drag_threshold) ||
-              (ABS (point->press_x - motion_x) >= drag_threshold))
-            {
-              priv->in_gesture = TRUE;
-
-              if (!CLUTTER_GESTURE_ACTION_GET_CLASS (action)->gesture_prepare (action, actor))
-                {
-                  cancel_gesture (action);
-                  return CLUTTER_EVENT_PROPAGATE;
-                }
-
-              /* clutter_gesture_action_cancel() may have been called during
-               * gesture_prepare(), check that the gesture is still active. */
-              if (!priv->in_gesture)
-                return CLUTTER_EVENT_PROPAGATE;
-
-              g_signal_emit (action, gesture_signals[GESTURE_BEGIN], 0, actor,
-                             &return_value);
-              if (!return_value)
-                {
-                  cancel_gesture (action);
-                  return CLUTTER_EVENT_PROPAGATE;
-                }
-            }
-          else
+          if (!begin_gesture(action, actor))
             return CLUTTER_EVENT_PROPAGATE;
         }
 
+      clutter_event_free (point->last_event);
+      point->last_event = clutter_event_copy (event);
+
       point->last_delta_x = motion_x - point->last_motion_x;
       point->last_delta_y = motion_y - point->last_motion_y;
       point->last_motion_x = motion_x;
@@ -325,6 +358,16 @@ stage_captured_event_cb (ClutterActor       *stage,
           cancel_gesture (action);
           return CLUTTER_EVENT_PROPAGATE;
         }
+
+      /* Check if a _TRIGGER_EDGE_BEFORE gesture needs to be cancelled because
+       * the drag threshold has been exceeded. */
+      if (priv->edge == CLUTTER_GESTURE_TRIGGER_EDGE_BEFORE &&
+          ((fabsf (point->press_y - motion_y) > drag_threshold) ||
+           (fabsf (point->press_x - motion_x) > drag_threshold)))
+        {
+          cancel_gesture (action);
+          return CLUTTER_EVENT_PROPAGATE;
+        }
       break;
 
     case CLUTTER_BUTTON_RELEASE:
@@ -332,6 +375,9 @@ stage_captured_event_cb (ClutterActor       *stage,
       {
         clutter_event_get_coords (event, &point->release_x, &point->release_y);
 
+        clutter_event_free (point->last_event);
+        point->last_event = clutter_event_copy (event);
+
         if (priv->in_gesture &&
             ((priv->points->len - 1) < priv->requested_nb_points))
           {
@@ -400,6 +446,11 @@ actor_captured_event_cb (ClutterActor *actor,
                               G_CALLBACK (stage_captured_event_cb),
                               action);
 
+  /* Start the gesture immediately if the gesture has no
+   * _TRIGGER_EDGE_AFTER drag threshold. */
+  if (priv->edge != CLUTTER_GESTURE_TRIGGER_EDGE_AFTER)
+    begin_gesture (action, actor);
+
   return CLUTTER_EVENT_PROPAGATE;
 }
 
@@ -448,6 +499,26 @@ default_event_handler (ClutterGestureAction *action,
   return TRUE;
 }
 
+
+/*< private >
+ * _clutter_gesture_action_set_threshold_trigger_edge:
+ * @action: a #ClutterGestureAction
+ * @edge: the %ClutterGestureTriggerEdge
+ *
+ * Sets the edge trigger for the gesture drag threshold, if any.
+ *
+ * This function can be called by #ClutterGestureAction subclasses that needs
+ * to change the %CLUTTER_GESTURE_TRIGGER_EDGE_AFTER default.
+ *
+ * Since: 1.14
+ */
+void
+_clutter_gesture_action_set_threshold_trigger_edge  (ClutterGestureAction     *action,
+                                                     ClutterGestureTriggerEdge edge)
+{
+  action->priv->edge = edge;
+}
+
 static void
 clutter_gesture_action_class_init (ClutterGestureActionClass *klass)
 {
@@ -561,7 +632,10 @@ clutter_gesture_action_init (ClutterGestureAction *self)
                                             ClutterGestureActionPrivate);
 
   self->priv->points = g_array_sized_new (FALSE, TRUE, sizeof (GesturePoint), 3);
+  g_array_set_clear_func (self->priv->points, (GDestroyNotify) gesture_point_unset);
+
   self->priv->requested_nb_points = 1;
+  self->priv->edge = CLUTTER_GESTURE_TRIGGER_EDGE_AFTER;
 }
 
 /**
@@ -582,75 +656,78 @@ clutter_gesture_action_new (void)
 /**
  * clutter_gesture_action_get_press_coords:
  * @action: a #ClutterGestureAction
- * @device: currently unused, set to 0
+ * @point: the touch point index, with 0 being the first touch
+ *   point received by the action
  * @press_x: (out) (allow-none): return location for the press
  *   event's X coordinate
  * @press_y: (out) (allow-none): return location for the press
  *   event's Y coordinate
  *
  * Retrieves the coordinates, in stage space, of the press event
- * that started the dragging for an specific pointer device
+ * that started the dragging for a specific touch point.
  *
  * Since: 1.8
  */
 void
 clutter_gesture_action_get_press_coords (ClutterGestureAction *action,
-                                         guint                 device,
+                                         guint                 point,
                                          gfloat               *press_x,
                                          gfloat               *press_y)
 {
   g_return_if_fail (CLUTTER_IS_GESTURE_ACTION (action));
-  g_return_if_fail (action->priv->points->len > device);
+  g_return_if_fail (action->priv->points->len > point);
 
   if (press_x)
     *press_x = g_array_index (action->priv->points,
                               GesturePoint,
-                              device).press_x;
+                              point).press_x;
 
   if (press_y)
     *press_y = g_array_index (action->priv->points,
                               GesturePoint,
-                              device).press_y;
+                              point).press_y;
 }
 
 /**
  * clutter_gesture_action_get_motion_coords:
  * @action: a #ClutterGestureAction
- * @device: currently unused, set to 0
+ * @point: the touch point index, with 0 being the first touch
+ *   point received by the action
  * @motion_x: (out) (allow-none): return location for the latest motion
  *   event's X coordinate
  * @motion_y: (out) (allow-none): return location for the latest motion
  *   event's Y coordinate
  *
  * Retrieves the coordinates, in stage space, of the latest motion
- * event during the dragging
+ * event during the dragging.
  *
  * Since: 1.8
  */
 void
 clutter_gesture_action_get_motion_coords (ClutterGestureAction *action,
-                                          guint                 device,
+                                          guint                 point,
                                           gfloat               *motion_x,
                                           gfloat               *motion_y)
 {
   g_return_if_fail (CLUTTER_IS_GESTURE_ACTION (action));
-  g_return_if_fail (action->priv->points->len > device);
+  g_return_if_fail (action->priv->points->len > point);
 
   if (motion_x)
     *motion_x = g_array_index (action->priv->points,
                                GesturePoint,
-                               device).last_motion_x;
+                               point).last_motion_x;
 
   if (motion_y)
     *motion_y = g_array_index (action->priv->points,
                                GesturePoint,
-                               device).last_motion_y;
+                               point).last_motion_y;
 }
 
 /**
  * clutter_gesture_action_get_motion_delta:
  * @action: a #ClutterGestureAction
- * @device: currently unused, set to 0
+ * @point: the touch point index, with 0 being the first touch
+ *   point received by the action
  * @delta_x: (out) (allow-none): return location for the X axis
  *   component of the incremental motion delta
  * @delta_y: (out) (allow-none): return location for the Y axis
@@ -665,21 +742,21 @@ clutter_gesture_action_get_motion_coords (ClutterGestureAction *action,
  */
 gfloat
 clutter_gesture_action_get_motion_delta (ClutterGestureAction *action,
-                                         guint                 device,
+                                         guint                 point,
                                          gfloat               *delta_x,
                                          gfloat               *delta_y)
 {
   gfloat d_x, d_y;
 
   g_return_val_if_fail (CLUTTER_IS_GESTURE_ACTION (action), 0);
-  g_return_val_if_fail (action->priv->points->len > device, 0);
+  g_return_val_if_fail (action->priv->points->len > point, 0);
 
   d_x = g_array_index (action->priv->points,
                        GesturePoint,
-                       device).last_delta_x;
+                       point).last_delta_x;
   d_y = g_array_index (action->priv->points,
                        GesturePoint,
-                       device).last_delta_y;
+                       point).last_delta_y;
 
   if (delta_x)
     *delta_x = d_x;
@@ -693,54 +770,56 @@ clutter_gesture_action_get_motion_delta (ClutterGestureAction *action,
 /**
  * clutter_gesture_action_get_release_coords:
  * @action: a #ClutterGestureAction
- * @device: currently unused, set to 0
+ * @point: the touch point index, with 0 being the first touch
+ *   point received by the action
  * @release_x: (out) (allow-none): return location for the X coordinate of
  *   the last release
  * @release_y: (out) (allow-none): return location for the Y coordinate of
  *   the last release
  *
- * Retrieves the coordinates, in stage space, of the point where the pointer
- * device was last released.
+ * Retrieves the coordinates, in stage space, where the touch point was
+ * last released.
  *
  * Since: 1.8
  */
 void
 clutter_gesture_action_get_release_coords (ClutterGestureAction *action,
-                                           guint                 device,
+                                           guint                 point,
                                            gfloat               *release_x,
                                            gfloat               *release_y)
 {
   g_return_if_fail (CLUTTER_IS_GESTURE_ACTION (action));
-  g_return_if_fail (action->priv->points->len > device);
+  g_return_if_fail (action->priv->points->len > point);
 
   if (release_x)
     *release_x = g_array_index (action->priv->points,
                                 GesturePoint,
-                                device).release_x;
+                                point).release_x;
 
   if (release_y)
     *release_y = g_array_index (action->priv->points,
                                 GesturePoint,
-                                device).release_y;
+                                point).release_y;
 }
 
 /**
  * clutter_gesture_action_get_velocity:
  * @action: a #ClutterGestureAction
- * @device: currently unused, set to 0
+ * @point: the touch point index, with 0 being the first touch
+ *   point received by the action
  * @velocity_x: (out) (allow-none): return location for the latest motion
  *   event's X velocity
  * @velocity_y: (out) (allow-none): return location for the latest motion
  *   event's Y velocity
  *
- * Retrieves the velocity, in stage pixels per milliseconds, of the
- * latest motion event during the dragging
+ * Retrieves the velocity, in stage pixels per millisecond, of the
+ * latest motion event during the dragging.
  *
  * Since: 1.12
  */
 gfloat
 clutter_gesture_action_get_velocity (ClutterGestureAction *action,
-                                     guint                 device,
+                                     guint                 point,
                                      gfloat               *velocity_x,
                                      gfloat               *velocity_y)
 {
@@ -748,14 +827,14 @@ clutter_gesture_action_get_velocity (ClutterGestureAction *action,
   gint64 d_t;
 
   g_return_val_if_fail (CLUTTER_IS_GESTURE_ACTION (action), 0);
-  g_return_val_if_fail (action->priv->points->len > device, 0);
+  g_return_val_if_fail (action->priv->points->len > point, 0);
 
-  distance = clutter_gesture_action_get_motion_delta (action, device,
+  distance = clutter_gesture_action_get_motion_delta (action, point,
                                                       &d_x, &d_y);
 
   d_t = g_array_index (action->priv->points,
                        GesturePoint,
-                       device).last_delta_time;
+                       point).last_delta_time;
 
   if (velocity_x)
     *velocity_x = d_t > FLOAT_EPSILON ? d_x / d_t : 0;
@@ -812,18 +891,15 @@ clutter_gesture_action_set_n_touch_points (ClutterGestureAction *action,
       if (priv->points->len < priv->requested_nb_points)
         cancel_gesture (action);
     }
-  else
+  else if (priv->edge == CLUTTER_GESTURE_TRIGGER_EDGE_AFTER)
     {
       if (priv->points->len >= priv->requested_nb_points)
         {
           ClutterActor *actor =
             clutter_actor_meta_get_actor (CLUTTER_ACTOR_META (action));
-          ClutterSettings *settings = clutter_settings_get_default ();
           gint i, drag_threshold;
 
-          g_object_get (settings,
-                        "dnd-drag-threshold", &drag_threshold,
-                        NULL);
+          drag_threshold = gesture_get_threshold (action);
 
           for (i = 0; i < priv->points->len; i++)
             {
@@ -832,16 +908,7 @@ clutter_gesture_action_set_n_touch_points (ClutterGestureAction *action,
               if ((ABS (point->press_y - point->last_motion_y) >= drag_threshold) ||
                   (ABS (point->press_x - point->last_motion_x) >= drag_threshold))
                 {
-                  gboolean return_value;
-
-                  priv->in_gesture = TRUE;
-
-                  g_signal_emit (action, gesture_signals[GESTURE_BEGIN], 0, actor,
-                                 &return_value);
-
-                  if (!return_value)
-                    cancel_gesture (action);
-
+                  begin_gesture (action, actor);
                   break;
                 }
             }
@@ -891,7 +958,8 @@ clutter_gesture_action_get_sequence (ClutterGestureAction *action,
 /**
  * clutter_gesture_action_get_device:
  * @action: a #ClutterGestureAction
- * @point: index of a point currently active
+ * @point: the touch point index, with 0 being the first touch
+ *   point received by the action
  *
  * Retrieves the #ClutterInputDevice of a touch point.
  *
@@ -909,6 +977,32 @@ clutter_gesture_action_get_device (ClutterGestureAction *action,
   return g_array_index (action->priv->points, GesturePoint, point).device;
 }
 
+/**
+ * clutter_gesture_action_get_last_event:
+ * @action: a #ClutterGestureAction
+ * @point: index of a point currently active
+ *
+ * Retrieves a reference to the last #ClutterEvent for a touch point. Call
+ * clutter_event_copy() if you need to store the reference somewhere.
+ *
+ * Return value: (transfer none): the last #ClutterEvent for a touch point.
+ *
+ * Since: 1.14
+ */
+const ClutterEvent *
+clutter_gesture_action_get_last_event (ClutterGestureAction *action,
+                                       guint                 point)
+{
+  GesturePoint *gesture_point;
+
+  g_return_val_if_fail (CLUTTER_IS_GESTURE_ACTION (action), NULL);
+  g_return_val_if_fail (action->priv->points->len > point, NULL);
+
+  gesture_point = &g_array_index (action->priv->points, GesturePoint, point);
+
+  return gesture_point->last_event;
+}
+
 /**
  * clutter_gesture_action_cancel:
  * @action: a #ClutterGestureAction
@@ -923,6 +1017,4 @@ clutter_gesture_action_cancel (ClutterGestureAction *action)
   g_return_if_fail (CLUTTER_IS_GESTURE_ACTION (action));
 
   cancel_gesture (action);
-
-  g_array_set_size (action->priv->points, 0);
 }
