@@ -30,9 +30,6 @@
 
 #include <clutter/clutter-types.h>
 #include <clutter/clutter-group.h>
-#include <clutter/clutter-color.h>
-#include <clutter/clutter-event.h>
-#include <cairo.h>
 
 G_BEGIN_DECLS
 
@@ -128,6 +125,9 @@ struct _ClutterPerspective
  * Fog settings used to create the depth cueing effect.
  *
  * Since: 0.6
+ *
+ * Deprecated: 1.10: The fog-related API in #ClutterStage has been
+ *   deprecated as well.
  */
 struct _ClutterFog
 {
@@ -202,6 +202,14 @@ void            clutter_stage_ensure_current                    (ClutterStage
 void            clutter_stage_ensure_viewport                   (ClutterStage          *stage);
 void            clutter_stage_ensure_redraw                     (ClutterStage          *stage);
 
+#ifdef CLUTTER_ENABLE_EXPERIMENTAL_API
+CLUTTER_AVAILABLE_IN_1_14
+void            clutter_stage_set_sync_delay                    (ClutterStage          *stage,
+                                                                 gint                   sync_delay);
+CLUTTER_AVAILABLE_IN_1_14
+void            clutter_stage_skip_sync_delay                   (ClutterStage          *stage);
+#endif
+
 G_END_DECLS
 
 #endif /* __CLUTTER_STAGE_H__ */
