@@ -30,27 +30,39 @@
 
 #include <glib.h>
 
+#include "clutter-wayland.h"
 #include "clutter-stage-wayland.h"
 #include "clutter-backend-wayland.h"
 #include "clutter-stage-window.h"
 #include "clutter-stage-private.h"
 #include "clutter-event-private.h"
+#include "clutter-wayland.h"
 #include <cogl/cogl.h>
+#include <cogl/cogl-wayland-client.h>
 
 static ClutterStageWindowIface *clutter_stage_window_parent_iface = NULL;
 
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
+#define clutter_stage_wayland_get_type _clutter_stage_wayland_get_type
+
 G_DEFINE_TYPE_WITH_CODE (ClutterStageWayland,
                          clutter_stage_wayland,
                          CLUTTER_TYPE_STAGE_COGL,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_STAGE_WINDOW,
                                                 clutter_stage_window_iface_init));
 
+static void
+handle_ping (void *data,
+             struct wl_shell_surface *shell_surface,
+             uint32_t serial)
+{
+  wl_shell_surface_pong(shell_surface, serial);
+}
+
 static void
 handle_configure (void *data,
                   struct wl_shell_surface *shell_surface,
-                  uint32_t timestamp,
                   uint32_t edges,
                   int32_t width,
                   int32_t height)
@@ -71,8 +83,17 @@ handle_configure (void *data,
   clutter_stage_ensure_viewport (stage_cogl->wrapper);
 }
 
+static void
+handle_popup_done (void *data,
+                   struct wl_shell_surface *shell_surface)
+{
+  /* XXX: Fill me in. */
+}
+
 static const struct wl_shell_surface_listener shell_surface_listener = {
+       handle_ping,
        handle_configure,
+       handle_popup_done,
 };
 
 static void
@@ -140,7 +161,10 @@ clutter_stage_wayland_set_fullscreen (ClutterStageWindow *stage_window,
        * attached
        */
       _clutter_stage_window_redraw (stage_window);
-      wl_shell_surface_set_fullscreen (stage_wayland->wayland_shell_surface);
+      wl_shell_surface_set_fullscreen (stage_wayland->wayland_shell_surface,
+                                       WL_SHELL_SURFACE_FULLSCREEN_METHOD_DEFAULT,
+                                       0,
+                                       NULL);
     }
   else
     {
@@ -160,8 +184,11 @@ clutter_stage_wayland_resize (ClutterStageWindow *stage_window,
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
 
   /* Resize preserving top left */
-  cogl_wayland_onscreen_resize (stage_cogl->onscreen, width, height, 0, 0);
-  _clutter_stage_window_redraw (stage_window);
+  if (stage_cogl->onscreen)
+    {
+      cogl_wayland_onscreen_resize (stage_cogl->onscreen, width, height, 0, 0);
+      _clutter_stage_window_redraw (stage_window);
+    }
 }
 
 static void
