@@ -239,12 +239,7 @@ clutter_backend_x11_create_device_manager (ClutterBackendX11 *backend_x11)
             {
 #ifdef HAVE_XINPUT_2
               int major = 2;
-
-#ifdef HAVE_XINPUT_2_2
-              int minor = 2;
-#else
-              int minor = 0;
-#endif /* HAVE_XINPUT_2_2 */
+              int minor = 3;
 
               if (XIQueryVersion (backend_x11->xdpy, &major, &minor) != BadRequest)
                 {
@@ -395,10 +390,6 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
 
   settings = clutter_settings_get_default ();
 
-  /* Cogl needs to know the Xlib display connection for
-     CoglTexturePixmapX11 */
-  cogl_xlib_set_display (backend_x11->xdpy);
-
   /* add event filter for Cogl events */
   clutter_x11_add_filter (cogl_xlib_filter, NULL);
 
