@@ -94,13 +94,13 @@ translate_valuator_class (Display             *xdisplay,
       atoms_initialized = TRUE;
     }
 
-  for (i = CLUTTER_INPUT_AXIS_IGNORE;
-       i < CLUTTER_INPUT_AXIS_LAST;
+  for (i = 0;
+       i < N_AXIS_ATOMS;
        i += 1)
     {
       if (clutter_input_axis_atoms[i] == class->label)
         {
-          axis = i;
+          axis = i + 1;
           break;
         }
     }
@@ -745,7 +745,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
           }
 
         if (source_device)
-          _clutter_input_device_reset_scroll_info (device);
+          _clutter_input_device_reset_scroll_info (source_device);
       }
       retval = CLUTTER_TRANSLATE_REMOVE;
       break;
@@ -1345,8 +1345,8 @@ relate_slaves (gpointer key,
   ClutterDeviceManagerXI2 *manager_xi2 = data;
   ClutterInputDevice *master, *slave;
 
-  master = g_hash_table_lookup (manager_xi2->devices_by_id, key);
-  slave = g_hash_table_lookup (manager_xi2->devices_by_id, value);
+  slave = g_hash_table_lookup (manager_xi2->devices_by_id, key);
+  master = g_hash_table_lookup (manager_xi2->devices_by_id, value);
 
   _clutter_input_device_set_associated_device (slave, master);
   _clutter_input_device_add_slave (master, slave);
